-- Test check_cookie
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(21);

set search_path to auth, tipus, public;

select has_function('public', 'check_cookie', array ['text']);
select function_lang_is('public', 'check_cookie', array ['text'], 'plpgsql');
select function_returns('public', 'check_cookie', array ['text'], 'name');
select is_definer('public', 'check_cookie', array ['text']);
select volatility_is('public', 'check_cookie', array ['text'], 'stable');
select function_privs_are('public', 'check_cookie', array ['text'], 'guest', array []::text[]);
select function_privs_are('public', 'check_cookie', array ['text'], 'publisher', array []::text[]);
select function_privs_are('public', 'check_cookie', array ['text'], 'admin', array []::text[]);
select function_privs_are('public', 'check_cookie', array ['text'], 'authenticator', array ['EXECUTE']);

set client_min_messages to warning;
truncate auth."user" cascade;
reset client_min_messages;

insert into auth."user" (user_id, email, name, password, role, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', 'publisher', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e',
        current_timestamp + interval '1 month')
     , (9, 'admin@tandem.blog', 'Demo', 'test', 'admin', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524',
        current_timestamp + interval '1 month')
;

prepare user_info as
    select current_user_email(), current_user_cookie();

select is(
               check_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog'),
               'publisher'::name,
               'Should validate the cookie for the first user'
           );

select results_eq(
               'user_info',
               $$ values ('demo@tandem.blog', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e') $$,
               'Should have updated the settings with the user info'
           );

select is(
               check_cookie('12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524/admin@tandem.blog'),
               'admin'::name,
               'Should validate the cookie for the second user'
           );

select results_eq(
               'user_info',
               $$ values ('admin@tandem.blog', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524') $$,
               'Should have updated the settings with the other user info'
           );

select is(
               check_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/admin@tandem.blog'),
               'guest'::name,
               'Should only match with the correct email'
           );

select results_eq(
               'user_info',
               $$ values ('', '') $$,
               'Should have updated the settings with a guest user'
           );

select is(
               check_cookie('aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/admin@tandem.blog'),
               'guest'::name,
               'Should only match with the correct cookie value'
           );

select results_eq(
               'user_info',
               $$ values ('', '') $$,
               'Should have left the settings with a guest user'
           );

update "user"
set cookie_expires_at = current_timestamp - interval '1 minute';

select is(
               check_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog'),
               'guest'::name,
               'Should not allow expired cookies'
           );

select results_eq(
               'user_info',
               $$ values ('', '') $$,
               'Should have left the settings with a guest user'
           );

select is(
               check_cookie('12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524/admin@tandem.blog'),
               'guest'::name,
               'Should not allow expired cookied for the other user as well'
           );

select results_eq(
               'user_info',
               $$ values ('', '') $$,
               'Should have left the settings with a guest user'
           );


select *
from finish();

rollback;
